#
# spec file
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor serial%{nil}

%define so_ver 2

%if 0%{?sle_version} >= 150200
%define DisOMPI1 ExclusiveArch:  do_not_build
%endif
%if !0%{?is_opensuse} && 0%{?sle_version:1} && 0%{?sle_version} < 150200
%define DisOMPI3 ExclusiveArch:  do_not_build
%endif

%if "%flavor" == ""
ExclusiveArch:  do_not_build
%endif

%bcond_without openblas

%if "%flavor" == "serial"
%{bcond_with hpc}
%endif

%if "%flavor" == "openmpi4"
%define mpi_family openmpi
%define mpi_ver 4
%{bcond_with hpc}
%endif

%if "%flavor" == "openmpi5"
%define mpi_family openmpi
%define mpi_ver 5
%{bcond_with hpc}
%endif

# openmpi 1 was called just "openmpi" in Leap 15.x/SLE15
%if 0%{?suse_version} >= 1550 || "%{mpi_family}" != "openmpi"  || "%{mpi_ver}" != "1"
%define mpi_ext %{?mpi_ver}
%endif

%if 0%{?mpi_family:1}
%{bcond_without mpi}
%define pkgname   parpack-%{mpi_family}%{?mpi_ext}
%define libname() libparpack%{so_ver}-%{mpi_family}%{?mpi_ext}
%global my_prefix %{_libdir}/mpi/gcc/%{mpi_family}%{?mpi_ext}
%global my_libdir %{my_prefix}/%{_lib}
%global my_incdir %{my_prefix}/include
%else
%{bcond_with mpi}
%define pkgname   arpack-ng
%define libname() libarpack%{so_ver}
%global my_prefix %{_prefix}
%global my_libdir %{_libdir}
%global my_incdir %{_includedir}
%endif

# 3. OOM on i586 and 32-bit Arm: https://github.com/opencollab/arpack-ng/issues/289
%ifarch i586 %{arm}
%bcond_with pyarpack
%else
# 2. Boost too old for 15.2 and earlier
%if 0%{?suse_version} < 1550
%bcond_with pyarpack
%else
# 1. Build python module once: for serial flavor only
%if %{with mpi}
%bcond_with pyarpack
%else
%bcond_without pyarpack
%endif
# /1
%endif
# /2
%endif
# /3

Name:           %{pkgname}
Version:        3.9.0
Release:        2.2
Summary:        Fortran77 subroutines for solving large scale eigenvalue problems
License:        BSD-3-Clause
Group:          System/Libraries
URL:            https://github.com/opencollab/arpack-ng
Source0:        https://github.com/opencollab/arpack-ng/archive/%{version}.tar.gz#/arpack-ng-%{version}.tar.gz
# PATCH-FEATURE-OPENSUSE arpack-ng-python-module-installdir.patch badshah400@gmail.com -- Install python module to standard python sitearch instead of libdir
Patch0:         arpack-ng-python-module-installdir.patch
%if %{with mpi}
BuildRequires:  %{mpi_family}%{?mpi_ext}-devel
%endif
BuildRequires:  blas-devel
BuildRequires:  cmake >= 3.12
BuildRequires:  gcc-c++
BuildRequires:  gcc-fortran
BuildRequires:  lapack-devel
%if %{with openblas}
BuildRequires:  libopenblas_pthreads-devel
%endif
BuildRequires:  libtool
BuildRequires:  pkg-config
BuildRequires:  pkgconfig(eigen3)
%if %{with pyarpack}
BuildRequires:  libboost_numpy3-devel
BuildRequires:  libboost_python3-devel
BuildRequires:  python3-devel
BuildRequires:  python3-numpy
%endif

%description
ARPACK is a collection of Fortran77 subroutines designed to solve
large scale eigenvalue problems.

Arpack-ng is the successor of the legacy Arpack. It is fully compatible
with Arpack.

%package        devel
Summary:        Development files for %{name}
Group:          Development/Libraries/Other
%if %{with mpi}
Requires:       %{mpi_family}%{?mpi_ext}-devel
%else
Obsoletes:      arpack-devel < %{version}
Provides:       arpack-devel = %{version}
%endif
Requires:       %{libname} = %{version}-%{release}
Requires:       blas-devel
Requires:       gcc-fortran
Requires:       lapack-devel
%if "%{name}" == "parpack-openmpi1"
Provides:       parpack-openmpi-devel = %{version}
%endif

%description    devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package     -n %{libname}
Summary:        Files needed for developing arpack based applications
Group:          System/Libraries
%if %{with mpi}
Requires:       %{mpi_family}%{?mpi_ext}-libs
%endif

%description -n %{libname}
ARPACK is a collection of Fortran77 subroutines designed to solve
large scale eigenvalue problems. This package contains the so
library links used for building arpack based applications.

%package -n python3-%{name}
Summary:        Python bindings for ARPACK
Group:          Development/Libraries/Python

%description -n python3-%{name}
ARPACK is a collection of Fortran77 subroutines designed to solve
large scale eigenvalue problems. This package provides the python
bindings for ARPACK.

%prep
%autosetup -p1 -n arpack-ng-%{version}

# create baselibs.conf based on flavor
cat >  %{_sourcedir}/baselibs.conf <<EOF
%{libname}
%{name}-devel
    requires -%{name}-<targettype>
    requires "%{libname}-<targettype> = <version>"
EOF

%build
# Force -fPIC, otherwise linking of the test binaries fails
# on aarch64 an ppc64
export FFLAGS="%{optflags} -fPIC"
export FCFLAGS="%{optflags} -fPIC"
export CFLAGS="%{optflags} -fPIC"
export CXXFLAGS="%{optflags} -fPIC"

%if %{with mpi}
source %{my_prefix}/bin/mpivars.sh
export CC=%{my_prefix}/bin/mpicc
export CXX=%{my_prefix}/bin/mpic++
export F77=%{my_prefix}/bin/mpif77
export MPIF77=%{my_prefix}/bin/mpif77
export LD_LIBRARY_PATH=%{my_prefix}/%{_lib}
%endif

%cmake \
  -DCMAKE_INSTALL_PREFIX:PATH=%{my_prefix} \
  -DCMAKE_INSTALL_LIBDIR:PATH=%{my_libdir} \
  -DCMAKE_SKIP_RPATH:BOOL=OFF \
  -DCMAKE_SKIP_INSTALL_RPATH:BOOL=ON \
  -DCMAKE_CXX_COMPILER_VERSION=$(gcc -dumpfullversion) \
  -DMPI:BOOL=%{?with_mpi:ON}%{!?with_mpi:OFF} \
  -DPYTHON3:BOOL=%{?with_pyarpack:ON}%{!?with_pyarpack:OFF}

%if %{with pyarpack}
# Build pyarpack in a multiple steps, as arpackmm and
# pyarpack need a considerable amount of memory
%cmake_build arpack
%cmake_build arpackmm
%cmake_build pyarpack
%endif
# Make sure all (remaining) targets are build
%cmake_build

%install
%cmake_install
find %{buildroot} -type f -name "*.la" -delete -print

# Remove sequential version files
%if %{with mpi}
rm -Rf %{buildroot}%{my_libdir}/pkgconfig
rm %{buildroot}%{my_libdir}/libarpack.so*
%endif

ln -s EXAMPLES examples

%check
%if %{with mpi}
source %{my_prefix}/bin/mpivars.sh
%endif
%ctest

%post   -n %{libname}           -p /sbin/ldconfig
%postun -n %{libname}           -p /sbin/ldconfig

%files -n %{libname}
%license COPYING
%{my_libdir}/*.so.*

%files devel
%doc examples
%doc CHANGES README.md
%{my_libdir}/*.so
%{my_incdir}/arpack-ng
%if %{without mpi}
%{_libdir}/pkgconfig/*.pc
%else
%dir %{my_libdir}/cmake
%endif
%{my_libdir}/cmake/arpackng

%if %{with pyarpack}
%files -n python3-%{name}
%{python3_sitearch}/*.so
%endif

%changelog
* Mon Oct 23 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for obsolete openmpi[123]
- Prepare support for openmpi5
* Wed Feb 15 2023 Ferdinand Thiessen <rpm@fthiessen.de>
- Update to version 3.9.0
  * parpack.h and parpack.hpp: type of rwork should be real instead
    of complex.
  * Allow ritz_option {"LR", "SR", "LI", "SI"} for complex eigenvalue
    problems in ICB.
  * Replace deprecated NP complex
  * Include location of header files changed to arpack-ng
  * Many smaller bugfixes
- Drop upsteam fixed and merged patches
  * fix_tautological_compare_321.patch
  * replace_deprecated_np-complex.patch
  * Use-CMAKE_INSTALL_FULL_-dir.patch
- Rebased arpack-ng-python-module-installdir.patch
* Thu Jan 26 2023 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Fix build after numpy 1.24 API changes, add
  replace_deprecated_np-complex.patch
* Mon Dec  5 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Add memory constraints for pyarpack builds on s390x (slightly
  higher than on other 64 bit archs).
- Remove obsolete rpmlintrc.
* Tue Apr 26 2022 Jan Engelhardt <jengelh@inai.de>
- Add an rpmlintrc for shlib-policy-name-error
* Wed Feb  2 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Add fix_tautological_compare_321.patch
- Apply memory constraints also on ppc64/ppc64le
* Wed Nov 24 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Disable pyarpack on armv6/7 to fix build
* Thu Sep 30 2021 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Reduce memory requirements/constraints:
  * Only apply to serial multibuild flavor, where pyarpack is built
  * Build arpackmm and pyarpack sequentially
  * Use memory instead of physicalmemory
- Fix pkgconfig files, add Use-CMAKE_INSTALL_FULL_-dir.patch
* Thu Sep 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Increase RAM size required in _constraints to avoid OOM seen in
  Tumbleweed
* Thu Jul  1 2021 Andreas Schwab <schwab@suse.de>
- Add option to build without openblas
- Adjust constraints for riscv64
* Thu Jun 10 2021 Atri Bhattacharya <badshah400@gmail.com>
- Add openmpi4 to the list of multibuild flavors (appropriate
  definitions were already in spec file).
* Fri May 28 2021 Ludwig Nussel <lnussel@suse.de>
- don't own /usr/lib/pkgconfig, already owned by filesystem
* Thu Dec 10 2020 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 3.8.0:
  * Bug fixes:
  - bmat return "G" instead of "B" for generalized matrix in
    arpack.hpp.
  - pass arrays of chars as scalar in fortran calls in order not
    to crash when calling subroutines through icb interface.
  - fix 'Unknown CMake command "check_symbol_exists".' when
    ICB=ON.
  - fix arpackdef.h (resp. arpackicb.h) must be included only by
    C/C++ (resp. F77/F90).
  - iparam/ipntr sizes may change depending on cases.
  - ILP64 support: using debug_c and stat_c.
  - fix check precision which may fail with some ATLAS versions.
  - fix 'eval: Syntax error: "(" unexpected' error at build
    time.
  - ICB using rvec/select: rvec/select turned to integer bool
    should be, but, is not always supported (depend on compiler,
    options).
  * arpackmm:
  - extract arpackSolver.hpp from arpakmm.cpp.
  - arpackSolver/arpackmm: switch eigen version to 3.3.
  - arpackmm: add --slvItrPC option (PC: Jacobi, ILU).
  - arpackmm: add --slv LLT LDLT (for SPD matrices).
  - arpackmm: add --simplePrec option (to enable use of s*upd).
  - arpackmm: add --dense option.
  * pyarpack: python binding based on Boost.Python.Numpy exposing
    C++ API.
  * autotools: provide *.cmake files (in addition to *.pc file).
  * Only build shared libraries by default.  To build static
    libraries, use --enable-static (autotools) or
  - DBUILD_SHARED_LIBS=OFF (cmake).
  * [CLEAN] arpackSolver API: more convenient, suppress template
    parameters when possible.
  * Add parpack.pc and arpackSolver.pc.
  * Support of gfortran 10.
- Drop patches incorporated upstream:
  * arpack-ng-gcc10.patch.
  * arpack-ng-double-comparison.patch.
- New patches:
  * arpack-ng-python-module-installdir.patch to move python
    module to standard python sitearch.
- Enable pyarpack: a python interface for arpack-ng, and split it
  out into a new package: python3-arpack-ng; this module is only
  built once -- for the serial flavor:
  * Disabled for i586 (gh#opencollab/arpack-ng#289).
  * Disbaled for openSUSE < 1550: boost too old.
- Add _constraints to allow enough memory (12 GB) and disk size
  (3 GB) required to build pyarpack (only for x86_64, aarch64).
- Switch to building with cmake:
  * Add BuildRequires: cmake.
  * Pass -DCMAKE_INSTALL_<foo> options to suggest correct install
    paths for different flavors.
  * CMAKE_CXX_COMPILER_VERSION: GCC version is required to build
    pyarpack
  * To work around broken rpath handling in Leap 15.2's macros,
    pass `-DCMAKE_SKIP_RPATH=OFF -DCMAKE_SKIP_INSTALL_RPATH=ON`
    (do this on all distro versions as it doesn't hurt)
- Use arpack-ng-%%{version} as the naming format for the source
  tarball.
* Tue Jun 16 2020 Atri Bhattacharya <badshah400@gmail.com>
- Add arpack-ng-double-comparison.patch: Add a tolerance for
  testing float equivalence in TESTS/bug_79_double_complex.f;
  fixes build failure for i586 [gh#opencollab/arpack-ng#269].
* Sat Jun  6 2020 Atri Bhattacharya <badshah400@gmail.com>
- Add arpack-ng-gcc10.patch to fix build against GCC 10
  [gh#opencollab/arpack-ng#239, gh#opencollab/arpack-ng#245].
* Fri Jan  3 2020 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Update to version 3.7.0:
  * [BUG FIX] ICB: missing workev for *[ds]neupd (real+not-sym) =>
    API/ABI change for *[ds]neupd_c.
  * [BUG FIX] autotools - make distcheck: fix circular dependencies.
  * arpackmm: utility to test arpack with matrix market files.
  * ICB: add ILP64 support.
  - arpack includes arpackdef.h when/where needed:
  - #define provide c_int/a_int according to architecture.
  - all f90 who need to include "arpackdef.h" must be moved to F90.
    Note: by convention, F90 are preprocessed (f90 are not).
  - MPI does not support ILP64: integer*4 must be imposed in all
    calls involving MPI (f90 example/test code).
- Update to version 3.6.3:
  * Add Fortran common initialization (block data).
  * Give up forcing the initial residual to be in the range of the
    operator OP after a restart (Closes: #142).
- Update to version 3.6.2:
  * Install: move headers into a dedicated directory
    (local/include/arpack).  (Closes #126)
  * Add configuration summary.
  * Improve the flag detection. Hopefully fix the ppc64el and other
    archs issues in Debian
- Update to version 3.6.1:
  * Fix a regression on i386 and other archs (Closes #123)
- Update to version 3.6.0:
  * Add support for ISO_C_BINDING (Fortran 2003) for ARPACK,
    PARPACK (Fortran <-> C/C++).
  * Provide find_package for (cmake) users to find arpack-ng.
  * Rename pslamch to pslamch10 to avoid symbol collision with
    Scalapack 2.0.2 in MPI context.
  * Force the initial residual to be in the range of the operator
    OP in the standard case, too (Closes: #79).
  * Add coverage information to improve testing:
    https://coveralls.io/github/opencollab/arpack-ng
  * Add C++11 interface through arpack.hpp and parpack.hpp
  * Rewrite C++ examples / tests demonstrating new C++11 interface
  * Pre-C++11 interface available through arpack.h and parpack.h
* Sun Dec 15 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Create baselibs.conf on the fly, based on flavor.
* Sun Dec  8 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Use _multibuild for building serial and MPI versions, readd
  openmpi(1) for older distribution versions, add openmpi3.
- Reflect MPI flavor in package names
- Stop distributing compiled examples
* Fri Nov  1 2019 Atri Bhattacharya <badshah400@gmail.com>
- Fix file list from previous change: use /%%{ompi_ver}/ as the
  openmpi directory instead of the no longer existing /openmpi/.
* Thu Oct 10 2019 Nicolas Morey-Chaisemartin <nmoreychaisemartin@suse.com>
- Replace openmpi(1) by newer openmpi2
- Add explicit dependency to MPI library as multiple ones
  may provide the same symbols
* Sun Jul  9 2017 badshah400@gmail.com
- Update to version 3.5.0:
  * Improve cmake build system: disable C++ detection, set
    default build type.
  * Use AC_PROG_FC instead of AC_PROG_F77 for proper
    inizialization for the usage of AC_FC_LINE_LENGTH.
  * Improve cmake build system: add make install and fix shared
    libraries.
  * fix usages of DLACPY to not alias inputs
    (patch from
    https://software.intel.com/en-us/articles/how-to-resolve-arpack-issues-with-intel-mkl-110-update-3)
  * Fix possible deadlock when PARPACK call uses communicator
    with a larger number of CPUs than previous call
  * Portability improvements to the autotools build system.
  * Let cmake guess the default installation directories.  Can be
    overridden by changing CMAKE_INSTALL_LIBDIR and
    CMAKE_INSTALL_BINDIR.
  * Shared libraries built by cmake now have their SONAME set
    identical to those built by autotools.
  * Avoid purification stage in [d,s]neupd.f if it requires
    division by zero (Closes: gh#opencollab/arpack-ng#58).
* Fri Jul  8 2016 dmitry_r@opensuse.org
- Update to version 3.4.0
  * Build system updates
* Tue May 31 2016 ronisbr@gmail.com
- Update to version 3.3.0
  * Rename pdlamch to pdlamch10 to avoid symbol collision with
    Scalapack 2.0.2 in MPI context.
  * General improvements on the build system.
  * libparpack links against libarpack (instead of doing a static
    link).
  * Reverts using {d,s}lahqr from lapack 2.
  * Use dlahqr from lapack 3 instead of dlaqrb.
- Run spec-cleaner.
- Minor improvements in the .spec file.
* Sat Dec 13 2014 dmitry_r@opensuse.org
- Update to version 3.2.0
  * See included CHANGES file for details
  * Remove obsolete 0001-Do-not-install-test-binaries.patch
* Wed Jan  8 2014 badshah400@gmail.com
- Add baselibs.conf file for multiarch binaries.
* Tue Dec 10 2013 sweet_f_a@gmx.de
- Update to version 3.1.4
  * libparpack2: missing dependency on MPI:
    http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=718790
  * Replace LAPACK second function with ARPACK's own arscnd in PARPACK
  * Fix issue #1259 in DSEUPD and SSEUPD
    The Ritz vector purification step assumes workl(iq) still contains the
    original Q matrix. This is however overwritten by the call to xGEQR2
    earlier.
  * Use configure supplied blas and lapack in the pkg-config.
    Thanks to Ward Poelmans (Closes: #1320)
  * Switch to automake 1.14 + libtool 2.4.2.
    Thanks to Ward Poelmans (Closes: #1321)
  * dseupd routine may lead to a segmentation fault
    Thanks to Edouard Canot (Closes: #1323)
  * dsaupd and 'BE' option returns wrong eigenvalues for a SPD matrix
    Thanks to Edouard Canot (Closes: #1329)
- enable test suite
- add 0001-Do-not-install-test-binaries.patch to remove useless
  test binaries, this removes the whole default arpack-ng package
* Fri Sep  6 2013 dmitry_r@opensuse.org
- Update to version 3.1.3
  * Replace depcomp symlink with actual file.
  * Replace arpack.pc with proper autotooled arpack.pc.in
  * Add debug.h to TESTS/Makefile.am sources
  * "make dist" is functionnal
  * Also build the library "libparpacksrcblacs" (PARPACK/UTIL/BLACS/)
  * Wrong call to pdlamch was causing segfaults
  * Get rid of the mpif.h occurences in the source code
  * Configure detected built-in LAPACK and BLAS, but refused to use them
  * Fixed division by zero in smlnum by usind p[d,s]lamch instead of the
    serial
* Sun Aug 26 2012 scorot@free.fr
- fix source file Url
* Sun Aug 26 2012 scorot@free.fr
- fix borken obsoletes/provides in devel package
* Tue Jul 10 2012 scorot@free.fr
- version 3.1.1
  * Many bug fixes in the parpack lib. It is an old patch from
    upstream. Thanks to Viral Shah for pinging us on this subject.
    See the PARPACK_CHANGES file for the details.
  * Fixed a lack of appropriate bounds check in DNAUP2.
    Thanks to Pauli Virtanen for the patch (Closes: #632)
  * Update of the doc about TOL in dnaupd.
  * Reorder bug fixed when eigenvectors are requested and the
    resulting number of converged eigenvalues is less than the
    number requested. Patches from Tim Mitchell. (Closes: #664)
- spec file cleanup
* Thu Jun  7 2012 scorot@free.fr
- add missing requirements of blas and lapack in devel package
* Mon May 14 2012 scorot@free.fr
- Modify Requires of devel subpackages to match changes in lapack
  and blas packages
* Thu May 10 2012 scorot@free.fr
- Modify BuildRequires to match recent name changes of lapack and
  blas in project.
* Tue May  8 2012 scorot@free.fr
- link against reference blas and lapack instead of openblas
- build parallel parpack libs for openmpi and mvapich2
- add examples files in the doc of devel package
* Sun May  6 2012 scorot@free.fr
- spec file cleanup
- fix license flag
* Wed Mar 21 2012 scorot@free.fr
- add provides %%%%name in devel package
* Tue Feb 28 2012 scorot@free.fr
- add BuildRoot flag in order to fix build on SLE
- fix wrong library minor number
- fix wrong requirement in devel package
* Sun Feb 26 2012 scorot@free.fr
- initial package
- version 3.0.2
